@@ -93,6 +93,75 @@ def retrieve_hzt_prod(folder_out, start_time, end_time,pattern_type='shell'):
         
     return all_files
 
+def retrieve_hzt_RT(tstep):
+    
+    """ Retrieves the preprocessed HZT products
+        A version adapted to real time implementation
+        Only used in for the function retrieve_hzt_prod
+
+    Parameters
+    ----------
+    
+    tstep: datetime
+        directory where to store the unzipped files
+                
+    Returns
+    -------
+    A list containing all the filepaths of the retrieved files
+
+    """
+
+    # Get list of available files
+    folder_in = constants.FOLDER_ISO0
+    content_zip = np.array([c for c in os.listdir(folder_in) 
+                            if (len(c.split('.')) == 2) and (int(c.split('.')[-1])>=800)])
+    
+    # HZT files are produced once an hour
+    start_time = tstep.replace(minute=0)
+    end_time = start_time + datetime.timedelta(hours=1)
+
+    try:            
+        # Sort filelist to most recent prediction
+        content_filt = np.array([c for c in content_zip if c.endswith('800')])
+        times_filt = np.array([datetime.datetime.strptime(c[3:12],
+                            '%y%j%H%M')+datetime.timedelta(hours=int(c[-2::])) for c in content_filt])
+        conditions = np.array([np.logical_and((t >= start_time), (t <= end_time)) for t in times_filt])
+        
+        content_filt = content_filt[conditions]
+        times_filt = times_filt[conditions]
+    except:
+        logging.error('HZT data does not exist for '+start_time.strftime('%d-%b-%y'))
+        files = None
+        return
+        
+    # Check that an hourly estimate is available
+    all_hours = pd.date_range(start=start_time, end=end_time, freq='H')
+    
+    if len(all_hours) != len(times_filt):
+        content_times = np.array([datetime.datetime.strptime(c[3:12],
+                '%y%j%H%M')+datetime.timedelta(hours=int(c[-2::])) for c in content_zip])
+        # Find time that is missing:
+        for hh in all_hours:
+            if not hh in times_filt:
+                hh_last = np.where(hh==content_times)
+                times_filt = np.sort(np.append(times_filt, content_times[hh_last][-1]))
+                content_filt = np.sort(np.append(content_filt, content_zip[hh_last][-1]))
+    
+    # Get a list of all files to retrieve
+    conditions = np.array([np.logical_and(t >= start_time, t <= end_time)
+                        for t in times_filt])
+
+    if not np.any(conditions):
+        msg = '''
+        No file was found corresponding to this format, verify pattern and product_name
+        '''
+        raise ValueError(msg)
+        
+    files = sorted(np.array([folder_in + c for c in
+                            np.array(content_filt)[conditions]]))
+    
+    return files
+
 #-----------------------------------------------------------------------------------------
 def _retrieve_hzt_prod_daily(folder_out, start_time, end_time, pattern_type = 'shell'):
     
@@ -121,7 +190,7 @@ def _retrieve_hzt_prod_daily(folder_out, start_time, end_time, pattern_type = 's
     folder_out += '/'
     
     suffix =  str(start_time.year)[-2:] + str(start_time.timetuple().tm_yday).zfill(3)
-    folder_in = constants.FOLDER_RADAR + str(start_time.year) + '/' +  suffix + '/'
+    folder_in = constants.FOLDER_ISO0 + str(start_time.year) + '/' +  suffix + '/'
     name_zipfile = 'HZT'+ suffix+'.zip'
     
     try:
@@ -171,12 +240,11 @@ def _retrieve_hzt_prod_daily(folder_out, start_time, end_time, pattern_type = 's
             files_to_retrieve = files_to_retrieve.replace(fi,'')
             
     # Only unzip if at least one file does not exist
-    # (18 is the number of characters within the filename)
-    if len(files_to_retrieve) >= 18:
+    if len(files_to_retrieve.strip()) > 0:
         logging.info('Unzippping: '+ files_to_retrieve)   
-    cmd = 'unzip -j -o -qq "{:s}" {:s} -d {:s}'.format(folder_in + name_zipfile,
-         files_to_retrieve , folder_out)
-    subprocess.call(cmd, shell=True)
+        cmd = 'unzip -j -o -qq "{:s}" {:s} -d {:s}'.format(folder_in + name_zipfile,
+            files_to_retrieve , folder_out)
+        subprocess.call(cmd, shell=True)
         
     files = sorted(np.array([folder_out + c for c in
                             content_filt[conditions]]))    
@@ -323,12 +391,15 @@ def get_COSMO_variables(time, variables, sweeps = None, radar = None,
     # Round time to nearest hour
     t_near = round_to_hour(time)
     
+    cosmo_version = None
     if t_near < constants.COSMO1E_START:
         folder_cosmo = constants.FOLDER_COSMO1
         subfolder = ''
+        cosmo_version = '1'
     else:
         folder_cosmo = constants.FOLDER_COSMO1E
         subfolder = 'det/'
+        cosmo_version = '1e'
 
     # Get the closest COSMO-1 or 2 file in time
     grb = folder_cosmo + 'ANA{:s}/{:s}laf{:s}'.format(str(t_near.year)[2:],
@@ -355,7 +426,7 @@ def get_COSMO_variables(time, variables, sweeps = None, radar = None,
     # Interpolate for all radars and sweeps
     var_at_radar = {}
     for r in radar:
-        lut_rad = get_lookup('cosmo1_to_rad', r)
+        lut_rad = get_lookup('cosmo{:s}_to_rad'.format(cosmo_version), r)
         var_at_radar[r] = {}
         for v in variables:
             data = np.squeeze(file_COSMO.variables[v][:])
@@ -412,6 +483,14 @@ def retrieve_prod(folder_out, start_time, end_time, product_name,
     
     if product_name == 'ZZW' or product_name == 'ZZP': # no vpr for PPM and WEI
         product_name = 'ZZA'
+
+    if product_name == 'CPC':
+        folder_out = folder_out + '/CPC'
+    if product_name == 'CPCH':
+        folder_out = folder_out + '/CPCH'
+
+    if not os.path.exists(folder_out):
+        os.makedirs(folder_out)
     
     dt = datetime.timedelta(minutes = 5)
     delta = end_time - start_time
@@ -448,6 +527,66 @@ def retrieve_prod(folder_out, start_time, end_time, product_name,
             
     return all_files
 
+def retrieve_prod_RT(time, product_name, 
+                          pattern = None, pattern_type = 'shell', sweeps = None):
+    """ Adapted function from rainforest.common.retrieve_data
+        Here, it reads the data per timestep, and in the real-time
+        operation, the radar data is not zipped
+
+    Args:
+        time (datetime object): timestamp to extract
+        product_name (string): Name of the product to be extracted
+        sweeps (list): List of sweeps if not all want to be extracted. Defaults to None.
+
+    Raises:
+        ValueError: If no data is found
+        
+    Returns:
+        dict: dictionary containing with the the file list
+    """
+    
+    # Get all files
+    folder_radar = constants.FOLDER_RADAR
+    folder_in = folder_radar + product_name + '/'
+
+    # Get list of available files
+    content_zip = np.array(os.listdir(folder_in))
+
+    if pattern != None:
+        if pattern_type == 'shell':
+            content_zip = [c for c in content_zip 
+                        if fnmatch.fnmatch(os.path.basename(c), pattern)]
+        elif pattern_type == 'regex':
+            content_zip = [c for c in content_zip 
+                        if re.match(os.path.basename(c), pattern) != None]
+        else:
+            raise ValueError('Unknown pattern_type, must be either "shell" or "regex".')
+
+    # Derive datetime of each file
+    times_zip = np.array([datetime.datetime.strptime(c[3:12],
+                '%y%j%H%M') for c in content_zip])
+
+    # Get a list of all files to retrieve
+    conditions = (times_zip == time)
+    
+    # Filter on sweeps:
+    if sweeps != None:
+        sweeps_zip = np.array([int(c[-3:]) for c in content_zip])
+            # Get a list of all files to retrieve
+        conditions_sweep = np.array([s in sweeps for s in sweeps_zip])
+        conditions = np.logical_and(conditions, conditions_sweep)
+
+    if not np.any(conditions):
+        msg = '''
+        No file was found corresponding to this format, verify pattern and product_name
+        '''
+        raise ValueError(msg)
+    
+    files = sorted(np.array([folder_in + c for c in
+                            np.array(content_zip)[conditions]]))
+
+    return files   
+
 #-----------------------------------------------------------------------------------------
 def _retrieve_prod_daily(folder_out, start_time, end_time, product_name,
                   pattern = None, pattern_type = 'shell', sweeps = None):
@@ -511,8 +650,7 @@ def _retrieve_prod_daily(folder_out, start_time, end_time, product_name,
             files_to_retrieve = files_to_retrieve.replace(fi,'')
 
     # Only unzip if at least one file does not exist
-    # (18 is the number of characters within the filename)
-    if len(files_to_retrieve) >= 18:
+    if len(files_to_retrieve.strip()) > 0:
         cmd = 'unzip -j -o -qq "{:s}" {:s} -d {:s}'.format(folder_in + name_zipfile,
             files_to_retrieve , folder_out)
         subprocess.call(cmd, shell=True)
@@ -541,7 +679,7 @@ def retrieve_CPCCV(time, stations):
     A numpy array corresponding at the CPC.CV estimations at every specified 
     station
     """
-    
+
     year = time.year
 
     folder = constants.FOLDER_CPCCV + str(year) + '/'
@@ -552,7 +690,7 @@ def retrieve_CPCCV(time, stations):
         bname = os.path.basename(fname)
         times = bname.split('.')[1]
         tend = times.split('_')[1]
-        return datetime.datetime.strptime(tend,'%Y%m%d%H%M')
+        return datetime.datetime.strptime(tend,'%Y%m%d%H00')
 
     tend = np.array([_start_time(f) for f in files])
     
@@ -564,17 +702,78 @@ def retrieve_CPCCV(time, stations):
     
     data = io.read_xls(files[match[0]])
     
-    hour = int(datetime.datetime.strftime(time, '%Y%m%d%H%M'))
+    hour = int(datetime.datetime.strftime(time, '%Y%m%d%H00'))
     idx = np.where(np.array(data['time.stamp']) == hour)[0]
     data_hour = data.iloc[idx]
     data_hour_stations = data_hour.iloc[np.isin(np.array(data_hour['nat.abbr']), 
                                                 stations)]
     cpc_cv = []
+    cpc_xls = []
     for sta in stations:
         if sta in np.array(data_hour_stations['nat.abbr']):
             cpc_cv.append(float(data_hour_stations.loc[data_hour_stations['nat.abbr'] 
-                == sta]['CPC.CV']))
+                    == sta]['CPC.CV']))
+            cpc_xls.append(float(data_hour_stations.loc[data_hour_stations['nat.abbr']
+                    == sta]['CPC']))
         else:
             cpc_cv.append(np.nan)
-            
-    return np.array(cpc_cv)
+            cpc_xls.append(np.nan)
+
+    return np.array(cpc_cv), np.array(cpc_xls)
+
+#-----------------------------------------------------------------------------------------
+def retrieve_AQC_XLS(time, stations):
+    
+    """ Retrieves cross-validation CPC data for a set of stations from
+    the xls files prepared by Yanni
+
+    Parameters
+    ----------
+
+    time : datetime.datetime instance
+        starting time of the time range
+    stations : list of str
+        list of weather stations at which to retrieve the CPC.CV data
+    
+    Returns
+    -------
+    A numpy array corresponding at the CPC.CV estimations at every specified 
+    station
+    """
+
+    year = time.year
+
+    folder = constants.FOLDER_CPCCV + str(year) + '/'
+    
+    files = sorted([f for f in glob.glob(folder + '*.xls') if '.s' not in f])
+    
+    def _start_time(fname):
+        bname = os.path.basename(fname)
+        times = bname.split('.')[1]
+        tend = times.split('_')[1]
+        return datetime.datetime.strptime(tend,'%Y%m%d%H00')
+
+    tend = np.array([_start_time(f) for f in files])
+    
+    match = np.where(time < tend)[0]
+    
+    if not len(match):
+        logging.warn('Could not find CPC CV file for time {:s}'.format(time))
+        return np.zeros((len(stations))) + np.nan
+    
+    data = io.read_xls(files[match[0]])
+    
+    hour = int(datetime.datetime.strftime(time, '%Y%m%d%H00'))
+    idx = np.where(np.array(data['time.stamp']) == hour)[0]
+    data_hour = data.iloc[idx]
+    data_hour_stations = data_hour.iloc[np.isin(np.array(data_hour['nat.abbr']), 
+                                                stations)]
+    aqc_xls = []
+    for sta in stations:
+        if sta in np.array(data_hour_stations['nat.abbr']):
+            aqc_xls.append(float(data_hour_stations.loc[data_hour_stations['nat.abbr']
+                    == sta]['AQC']))
+        else:
+            aqc_xls.append(np.nan)
+
+    return np.array(aqc_xls)
\ No newline at end of file
